from switches import SwitchList
from commands import CommandList  # need to be called to initialize all enums
from analogvariables import AnalogVariableList
from digitalvariables import DigitalVariableList
from userpins import UserPinList
from entities.descriptors import PayloadFieldEnum


def generate_userpins_header(_userpins):

    includes = """"""

    ####################################################################################################################
    typedef_tpl = """
typedef struct {
[typedef_entries]
} userpins_t;
"""

    typedef_entry_tpl = """    enum_pinstate_t state_[name]; // status of usr pin [name], [desc]"""

    typedef_entries = list()
    for userpin in _userpins.entries():
        typedef_entry = typedef_entry_tpl
        typedef_entry = typedef_entry.replace('[name]', userpin.name)
        typedef_entry = typedef_entry.replace('[desc]', userpin.description)
        typedef_entries.append(typedef_entry)
    typedef = typedef_tpl
    typedef = typedef.replace('[typedef_entries]', '\n'.join(typedef_entries))

    ####################################################################################################################
    extern = """extern userpins_t userpins;"""

    ####################################################################################################################
    functionprotos_tpl = """
// control functions for user pin [name], [desc]
void userpin_[name]_set_high(void);
void userpin_[name]_set_low(void);
void userpin_[name]_set(enum_pinstate_t _state); 
enum_pinstate_t userpin_[name]_get(void); 
"""
    functionprotos_entries = list()
    for userpin in _userpins.entries():
        functionprotos_entry = functionprotos_tpl
        functionprotos_entry = functionprotos_entry.replace('[name]', userpin.name)
        functionprotos_entry = functionprotos_entry.replace('[desc]', userpin.description)
        functionprotos_entries.append(functionprotos_entry)
    functionprotos = '\n'.join(functionprotos_entries)

    ####################################################################################################################
    functions_by_enum_tpl = """
// return the user pin state set function by user pin enum
typedef void (*userpin_state_set_func_t)(enum_pinstate_t _state);
userpin_state_set_func_t userpin_get_function_set_by_enum(enum_pinnum_t _userpin_index);

// return the user pin state get function by user pin enum
typedef enum_pinstate_t (*userpin_state_get_func_t)(void);
userpin_state_get_func_t userpin_get_function_get_by_enum(enum_pinnum_t _userpin_index);
"""

    functions_by_enum = functions_by_enum_tpl

    return '\n'.join([includes, typedef, extern, functionprotos, functions_by_enum])


def generate_userpins_source(_userpins, _pin_enum):

    assert isinstance(_pin_enum, PayloadFieldEnum)

    includes = """"""

    ####################################################################################################################
    instance_tpl = """
userpins_t userpins = {
[instance_entries]
};
"""

    instance_entry_tpl = """    .state_[name] = PINSTATE_LOW"""

    instance_entries = list()
    for pin in _userpins.entries():
        instance_entry = instance_entry_tpl
        instance_entry = instance_entry.replace('[name]', pin.name)
        instance_entries.append(instance_entry)
    instance = instance_tpl
    instance = instance.replace('[instance_entries]', ',\n'.join(instance_entries))

    ####################################################################################################################
    functionimpls_tpl = """
// set pin [name] high and update userpins structure
void userpin_[name]_set_high(void) {
    [ioname]_SetHigh();
    userpins.state_[name] = PINSTATE_HIGH;
}

// set pin [name] low and update userpins structure
void userpin_[name]_set_low(void) {
    [ioname]_SetLow();
    userpins.state_[name] = PINSTATE_LOW;
}

// set state of userpin [name] and update userpins structure
void userpin_[name]_set(enum_pinstate_t _state) {
    if(_state == PINSTATE_HIGH) {
        userpin_[name]_set_high();
    }else{
        userpin_[name]_set_low();
    }
}

// get user pin [name] current status
enum_pinstate_t userpin_[name]_get(void) {
    return userpins.state_[name];
} 
"""

    functionimpls_entries = list()
    for pin in _userpins.entries():
        functionimpls_entry = functionimpls_tpl
        functionimpls_entry = functionimpls_entry.replace('[name]', pin.name)
        functionimpls_entry = functionimpls_entry.replace('[ioname]', pin.ioname)
        functionimpls_entries.append(functionimpls_entry)
    functionimpls = '\n'.join(functionimpls_entries)

    ####################################################################################################################
    functions_by_enum_tpl = """
// return the user pin state set function by user pin enum
userpin_state_set_func_t userpin_get_function_set_by_enum(enum_pinnum_t _userpin_index) {
[bodyset]
}

// return the user pin state get function by user pin enum
userpin_state_get_func_t userpin_get_function_get_by_enum(enum_pinnum_t _userpin_index) {
[bodyget]
}
"""

    bodyset = """    switch(_userpin_index) {\n"""
    bodyget = """    switch(_userpin_index) {\n"""
    enumobj = _pin_enum
    for idx in range(len(enumobj.value_list)):
        bodyset += '        case {}_{}: return userpin_{}_set;\n'.format(enumobj.name, enumobj.value_list[idx], _userpins.entries()[idx].name)
        bodyget += '        case {}_{}: return userpin_{}_get;\n'.format(enumobj.name, enumobj.value_list[idx], _userpins.entries()[idx].name)
    bodyset += '    }\n'
    bodyset += '    // this should never be reached\n'
    bodyset += '    error_die_forever();\n'
    bodyset += '    // but, just in case (if we neutralize the error function):\n'
    bodyset += '    return userpin_{}_set;'.format(_userpins.entries()[-1].name)
    bodyget += '    }\n'
    bodyget += '    // this should never be reached\n'
    bodyget += '    error_die_forever();\n'
    bodyget += '    // but, just in case (if we neutralize the error function):\n'
    bodyget += '    return userpin_{}_get;'.format(_userpins.entries()[-1].name)

    functions_by_enum = functions_by_enum_tpl
    functions_by_enum = functions_by_enum.replace('[bodyset]', bodyset)
    functions_by_enum = functions_by_enum.replace('[bodyget]', bodyget)

    return '\n'.join([includes, instance, functionimpls, functions_by_enum])


if __name__ == '__main__':
    userpins = UserPinList()
    analogvariables = AnalogVariableList()
    digitalvariables = DigitalVariableList()
    switches = SwitchList()
    commands = CommandList(switches, analogvariables, digitalvariables, userpins)
    pin_enum = PayloadFieldEnum.get_persistent_list()['PINNUM']

    print '\n' * 4
    print '=' * 64
    print generate_userpins_header(userpins)
    print generate_userpins_source(userpins, pin_enum)
